div.discussion-list .pytis-form.list-view .summary { display: none; }
div.discussion-list .pytis-form.list-view h3 { 
    margin-top: 0;
    font-weight: normal;
    font-size: 1em;
    background-color: $color.frame-bg;
    border: solid $color.frame-border 0;
    border-width: 1px 0;
    position: relative;
    padding: 1px 0;
}
div.discussion-list .pytis-form.list-view div.list-item.group-start { 
    padding-top: 0;
}
div.discussion-list .pytis-form.list-view div.list-item .actions { 
    text-align: right;
    margin: 0;
    margin-top: 8px;
}
div.discussion-list .pytis-form.list-view div.list-item .actions button {
    padding: 3px 8px;
}

div.discussion-list div.list-item div.attachment-list {
    margin-top: 0.3em;
}
div.discussion-list div.list-item div.vote-summary {
    border-top: 1px solid;
    border-left: 0.3em solid;
    border-color:  #7e7;
    background-color: #dfd;
    padding: 0.1em;
    margin: 0.4em 0;
}
div.discussion-list div.list-item div.vote-summary.negative-result {
    border-color: #e77;
    background-color: #fdd;
}
div.discussion-list .pytis-form.edit-form { 
    margin: 1.2em 0;
    background-color: $color.frame-bg;
    border: 1px solid $color.frame-border;
    border-width: 1px 0;
    padding: 0.3em;
    white-space: nowrap;
}
div.discussion-list .pytis-form.edit-form div.group,
div.discussion-list .pytis-form.edit-form div.submit-buttons {
    margin: 0;
    padding: 0;
    background: none;
    border: none;
}
div.discussion-list .pytis-form.edit-form div.submit-buttons {
    padding: 0;
    text-align: right;
}
div.discussion-list .pytis-form.edit-form div.submit-buttons button {
    margin: 0 0 0 5px;
}
div.discussion-list .pytis-form.edit-form div.footer { display: none; }
