create table countries (
	country_id serial primary key,
	country char(2) unique not null
);

insert into countries (country) values ('AD');
insert into countries (country) values ('AE'); 
insert into countries (country) values ('AF'); 
insert into countries (country) values ('AG'); 
insert into countries (country) values ('AI'); 
insert into countries (country) values ('AL'); 
insert into countries (country) values ('AM'); 
insert into countries (country) values ('AO'); 
insert into countries (country) values ('AQ'); 
insert into countries (country) values ('AR'); 
insert into countries (country) values ('AS'); 
insert into countries (country) values ('AT'); 
insert into countries (country) values ('AU'); 
insert into countries (country) values ('AW'); 
insert into countries (country) values ('AX'); 
insert into countries (country) values ('AZ'); 
insert into countries (country) values ('BA'); 
insert into countries (country) values ('BB'); 
insert into countries (country) values ('BD'); 
insert into countries (country) values ('BE'); 
insert into countries (country) values ('BF'); 
insert into countries (country) values ('BG'); 
insert into countries (country) values ('BH'); 
insert into countries (country) values ('BI'); 
insert into countries (country) values ('BJ'); 
insert into countries (country) values ('BL'); 
insert into countries (country) values ('BM'); 
insert into countries (country) values ('BN'); 
insert into countries (country) values ('BO'); 
insert into countries (country) values ('BQ'); 
insert into countries (country) values ('BR'); 
insert into countries (country) values ('BS'); 
insert into countries (country) values ('BT'); 
insert into countries (country) values ('BV'); 
insert into countries (country) values ('BW'); 
insert into countries (country) values ('BY'); 
insert into countries (country) values ('BZ'); 
insert into countries (country) values ('CA'); 
insert into countries (country) values ('CC'); 
insert into countries (country) values ('CD'); 
insert into countries (country) values ('CF'); 
insert into countries (country) values ('CG'); 
insert into countries (country) values ('CH'); 
insert into countries (country) values ('CI'); 
insert into countries (country) values ('CK'); 
insert into countries (country) values ('CL'); 
insert into countries (country) values ('CM'); 
insert into countries (country) values ('CN'); 
insert into countries (country) values ('CO'); 
insert into countries (country) values ('CR'); 
insert into countries (country) values ('CU'); 
insert into countries (country) values ('CV'); 
insert into countries (country) values ('CW'); 
insert into countries (country) values ('CX'); 
insert into countries (country) values ('CY'); 
insert into countries (country) values ('CZ'); 
insert into countries (country) values ('DE'); 
insert into countries (country) values ('DJ'); 
insert into countries (country) values ('DK'); 
insert into countries (country) values ('DM'); 
insert into countries (country) values ('DO'); 
insert into countries (country) values ('DZ'); 
insert into countries (country) values ('EC'); 
insert into countries (country) values ('EE'); 
insert into countries (country) values ('EG'); 
insert into countries (country) values ('EH'); 
insert into countries (country) values ('ER'); 
insert into countries (country) values ('ES'); 
insert into countries (country) values ('ET'); 
insert into countries (country) values ('FI'); 
insert into countries (country) values ('FJ'); 
insert into countries (country) values ('FK'); 
insert into countries (country) values ('FM'); 
insert into countries (country) values ('FO'); 
insert into countries (country) values ('FR'); 
insert into countries (country) values ('GA'); 
insert into countries (country) values ('GB'); 
insert into countries (country) values ('GD'); 
insert into countries (country) values ('GE'); 
insert into countries (country) values ('GF'); 
insert into countries (country) values ('GG'); 
insert into countries (country) values ('GH'); 
insert into countries (country) values ('GI'); 
insert into countries (country) values ('GL'); 
insert into countries (country) values ('GM'); 
insert into countries (country) values ('GN'); 
insert into countries (country) values ('GP'); 
insert into countries (country) values ('GQ'); 
insert into countries (country) values ('GR'); 
insert into countries (country) values ('GS'); 
insert into countries (country) values ('GT'); 
insert into countries (country) values ('GU'); 
insert into countries (country) values ('GW'); 
insert into countries (country) values ('GY'); 
insert into countries (country) values ('HK'); 
insert into countries (country) values ('HM'); 
insert into countries (country) values ('HN'); 
insert into countries (country) values ('HR'); 
insert into countries (country) values ('HT'); 
insert into countries (country) values ('HU'); 
insert into countries (country) values ('ID'); 
insert into countries (country) values ('IE'); 
insert into countries (country) values ('IL'); 
insert into countries (country) values ('IM'); 
insert into countries (country) values ('IN'); 
insert into countries (country) values ('IO'); 
insert into countries (country) values ('IQ'); 
insert into countries (country) values ('IR'); 
insert into countries (country) values ('IS'); 
insert into countries (country) values ('IT'); 
insert into countries (country) values ('JE'); 
insert into countries (country) values ('JM'); 
insert into countries (country) values ('JO'); 
insert into countries (country) values ('JP'); 
insert into countries (country) values ('KE'); 
insert into countries (country) values ('KG'); 
insert into countries (country) values ('KH'); 
insert into countries (country) values ('KI'); 
insert into countries (country) values ('KM'); 
insert into countries (country) values ('KN'); 
insert into countries (country) values ('KP'); 
insert into countries (country) values ('KR'); 
insert into countries (country) values ('KW'); 
insert into countries (country) values ('KY'); 
insert into countries (country) values ('KZ'); 
insert into countries (country) values ('LA'); 
insert into countries (country) values ('LB'); 
insert into countries (country) values ('LC'); 
insert into countries (country) values ('LI'); 
insert into countries (country) values ('LK'); 
insert into countries (country) values ('LR'); 
insert into countries (country) values ('LS'); 
insert into countries (country) values ('LT'); 
insert into countries (country) values ('LU'); 
insert into countries (country) values ('LV'); 
insert into countries (country) values ('LY'); 
insert into countries (country) values ('MA'); 
insert into countries (country) values ('MC'); 
insert into countries (country) values ('MD'); 
insert into countries (country) values ('ME'); 
insert into countries (country) values ('MF'); 
insert into countries (country) values ('MG'); 
insert into countries (country) values ('MH'); 
insert into countries (country) values ('MK'); 
insert into countries (country) values ('ML'); 
insert into countries (country) values ('MM'); 
insert into countries (country) values ('MN'); 
insert into countries (country) values ('MO'); 
insert into countries (country) values ('MP'); 
insert into countries (country) values ('MQ'); 
insert into countries (country) values ('MR'); 
insert into countries (country) values ('MS'); 
insert into countries (country) values ('MT'); 
insert into countries (country) values ('MU'); 
insert into countries (country) values ('MV'); 
insert into countries (country) values ('MW'); 
insert into countries (country) values ('MX'); 
insert into countries (country) values ('MY'); 
insert into countries (country) values ('MZ'); 
insert into countries (country) values ('NA'); 
insert into countries (country) values ('NC'); 
insert into countries (country) values ('NE'); 
insert into countries (country) values ('NF'); 
insert into countries (country) values ('NG'); 
insert into countries (country) values ('NI'); 
insert into countries (country) values ('NL'); 
insert into countries (country) values ('NO'); 
insert into countries (country) values ('NP'); 
insert into countries (country) values ('NR'); 
insert into countries (country) values ('NU'); 
insert into countries (country) values ('NZ'); 
insert into countries (country) values ('OM'); 
insert into countries (country) values ('PA'); 
insert into countries (country) values ('PE'); 
insert into countries (country) values ('PF'); 
insert into countries (country) values ('PG'); 
insert into countries (country) values ('PH'); 
insert into countries (country) values ('PK'); 
insert into countries (country) values ('PL'); 
insert into countries (country) values ('PM'); 
insert into countries (country) values ('PN'); 
insert into countries (country) values ('PR'); 
insert into countries (country) values ('PS'); 
insert into countries (country) values ('PT'); 
insert into countries (country) values ('PW'); 
insert into countries (country) values ('PY'); 
insert into countries (country) values ('QA'); 
insert into countries (country) values ('RE'); 
insert into countries (country) values ('RO'); 
insert into countries (country) values ('RS'); 
insert into countries (country) values ('RU'); 
insert into countries (country) values ('RW'); 
insert into countries (country) values ('SA'); 
insert into countries (country) values ('SB'); 
insert into countries (country) values ('SC'); 
insert into countries (country) values ('SD'); 
insert into countries (country) values ('SE'); 
insert into countries (country) values ('SG'); 
insert into countries (country) values ('SH'); 
insert into countries (country) values ('SI'); 
insert into countries (country) values ('SJ'); 
insert into countries (country) values ('SK'); 
insert into countries (country) values ('SL'); 
insert into countries (country) values ('SM'); 
insert into countries (country) values ('SN'); 
insert into countries (country) values ('SO'); 
insert into countries (country) values ('SR'); 
insert into countries (country) values ('SS'); 
insert into countries (country) values ('ST'); 
insert into countries (country) values ('SV'); 
insert into countries (country) values ('SX'); 
insert into countries (country) values ('SY'); 
insert into countries (country) values ('SZ'); 
insert into countries (country) values ('TC'); 
insert into countries (country) values ('TD'); 
insert into countries (country) values ('TF'); 
insert into countries (country) values ('TG'); 
insert into countries (country) values ('TH'); 
insert into countries (country) values ('TJ'); 
insert into countries (country) values ('TK'); 
insert into countries (country) values ('TL'); 
insert into countries (country) values ('TM'); 
insert into countries (country) values ('TN'); 
insert into countries (country) values ('TO'); 
insert into countries (country) values ('TR'); 
insert into countries (country) values ('TT'); 
insert into countries (country) values ('TV'); 
insert into countries (country) values ('TW'); 
insert into countries (country) values ('TZ'); 
insert into countries (country) values ('UA'); 
insert into countries (country) values ('UG'); 
insert into countries (country) values ('UM'); 
insert into countries (country) values ('US'); 
insert into countries (country) values ('UY'); 
insert into countries (country) values ('UZ'); 
insert into countries (country) values ('VA'); 
insert into countries (country) values ('VC'); 
insert into countries (country) values ('VE'); 
insert into countries (country) values ('VG'); 
insert into countries (country) values ('VI'); 
insert into countries (country) values ('VN'); 
insert into countries (country) values ('VU'); 
insert into countries (country) values ('WF'); 
insert into countries (country) values ('WS'); 
insert into countries (country) values ('YE'); 
insert into countries (country) values ('YT'); 
insert into countries (country) values ('ZA'); 
insert into countries (country) values ('ZM'); 
insert into countries (country) values ('ZW'); 
